PROTO = {
    'spaceone.identity.api.v1.domain': ['Domain'],
    'spaceone.identity.api.v1.domain_owner': ['DomainOwner'],
    'spaceone.identity.api.v1.endpoint': ['Endpoint'],
    'spaceone.identity.api.v1.provider': ['Provider'],
    'spaceone.identity.api.v1.service_account': ['ServiceAccount'],
    'spaceone.identity.api.v1.project_group': ['ProjectGroup'],
    'spaceone.identity.api.v1.project': ['Project'],
    'spaceone.identity.api.v1.policy': ['Policy'],
    'spaceone.identity.api.v1.role': ['Role'],
    'spaceone.identity.api.v1.role_binding': ['RoleBinding'],
    'spaceone.identity.api.v1.user': ['User'],
    'spaceone.identity.api.v1.api_key': ['APIKey'],
    'spaceone.identity.api.v1.token': ['Token'],
    'spaceone.identity.api.v1.authorization': ['Authorization']
}
